"use strict";
// *****************************************************************************
// Copyright (C) 2019 TypeFox and others.
//
// This program and the accompanying materials are made available under the
// terms of the Eclipse Public License v. 2.0 which is available at
// http://www.eclipse.org/legal/epl-2.0.
//
// This Source Code may also be made available under the following Secondary
// Licenses when the conditions for such availability set forth in the Eclipse
// Public License v. 2.0 are satisfied: GNU General Public License, version 2
// with the GNU Classpath Exception which is available at
// https://www.gnu.org/software/classpath/license.html.
//
// SPDX-License-Identifier: EPL-2.0 OR GPL-2.0 WITH Classpath-exception-2.0
// *****************************************************************************
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var IconThemeService_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.IconThemeService = exports.NoneIconTheme = void 0;
const inversify_1 = require("inversify");
const event_1 = require("../common/event");
const disposable_1 = require("../common/disposable");
const frontend_application_config_provider_1 = require("./frontend-application-config-provider");
let NoneIconTheme = class NoneIconTheme {
    constructor() {
        this.id = 'none';
        this.label = 'None';
        this.description = 'Disable file icons';
        this.hasFileIcons = true;
        this.hasFolderIcons = true;
        this.onDidChangeEmitter = new event_1.Emitter();
        this.onDidChange = this.onDidChangeEmitter.event;
        this.toDeactivate = new disposable_1.DisposableCollection();
    }
    activate() {
        if (this.toDeactivate.disposed) {
            this.toDeactivate.push(disposable_1.Disposable.create(() => this.fireDidChange()));
            this.fireDidChange();
        }
        return this.toDeactivate;
    }
    fireDidChange() {
        this.onDidChangeEmitter.fire({ affects: () => true });
    }
    canHandle() {
        if (this.toDeactivate.disposed) {
            return 0;
        }
        return Number.MAX_SAFE_INTEGER;
    }
    getIcon() {
        return '';
    }
};
NoneIconTheme = __decorate([
    (0, inversify_1.injectable)()
], NoneIconTheme);
exports.NoneIconTheme = NoneIconTheme;
let IconThemeService = IconThemeService_1 = class IconThemeService {
    constructor() {
        this.onDidChangeEmitter = new event_1.Emitter();
        this.onDidChange = this.onDidChangeEmitter.event;
        this._iconThemes = new Map();
        this.onDidChangeCurrentEmitter = new event_1.Emitter();
        this.onDidChangeCurrent = this.onDidChangeCurrentEmitter.event;
        this.toDeactivate = new disposable_1.DisposableCollection();
    }
    get ids() {
        return this._iconThemes.keys();
    }
    get definitions() {
        return this._iconThemes.values();
    }
    getDefinition(id) {
        return this._iconThemes.get(id);
    }
    init() {
        this.initDefaultTheme();
        this.register(this.noneIconTheme);
    }
    initDefaultTheme() {
        // If no theme is set yet, we store the desired default theme so it will be applied as soon as it is registered.
        // The registration may happen after preference initialization when contributed by a plugin.
        // If another theme is set by the user in the meantime, the local storage will be overridden anyway.
        if (!window.localStorage.getItem(IconThemeService_1.STORAGE_KEY) && frontend_application_config_provider_1.FrontendApplicationConfigProvider.get().defaultIconTheme) {
            window.localStorage.setItem(IconThemeService_1.STORAGE_KEY, frontend_application_config_provider_1.FrontendApplicationConfigProvider.get().defaultIconTheme);
        }
    }
    register(iconTheme) {
        if (this._iconThemes.has(iconTheme.id)) {
            console.warn(new Error(`Icon theme '${iconTheme.id}' has already been registered, skipping.`));
            return disposable_1.Disposable.NULL;
        }
        this._iconThemes.set(iconTheme.id, iconTheme);
        this.onDidChangeEmitter.fire(undefined);
        if (this.toDeactivate.disposed
            && window.localStorage.getItem(IconThemeService_1.STORAGE_KEY) === iconTheme.id) {
            this.setCurrent(iconTheme);
        }
        return disposable_1.Disposable.create(() => this.unregister(iconTheme.id));
    }
    unregister(id) {
        const iconTheme = this._iconThemes.get(id);
        if (!iconTheme) {
            return undefined;
        }
        this._iconThemes.delete(id);
        if (window.localStorage.getItem(IconThemeService_1.STORAGE_KEY) === id) {
            window.localStorage.removeItem(IconThemeService_1.STORAGE_KEY);
            this.onDidChangeCurrentEmitter.fire(this.default.id);
        }
        this.onDidChangeEmitter.fire(undefined);
        return iconTheme;
    }
    get current() {
        return this.getCurrent().id;
    }
    set current(id) {
        const newCurrent = this._iconThemes.get(id) || this.default;
        if (this.getCurrent().id !== newCurrent.id) {
            this.setCurrent(newCurrent);
        }
    }
    getCurrent() {
        const id = window.localStorage.getItem(IconThemeService_1.STORAGE_KEY);
        return id && this._iconThemes.get(id) || this.default;
    }
    setCurrent(current) {
        window.localStorage.setItem(IconThemeService_1.STORAGE_KEY, current.id);
        this.toDeactivate.dispose();
        this.toDeactivate.push(current.activate());
        this.onDidChangeCurrentEmitter.fire(current.id);
    }
    get default() {
        return this._iconThemes.get(frontend_application_config_provider_1.FrontendApplicationConfigProvider.get().defaultIconTheme) || this.fallback;
    }
    get fallback() {
        return this.noneIconTheme;
    }
    load() {
        return window.localStorage.getItem(IconThemeService_1.STORAGE_KEY) || undefined;
    }
};
IconThemeService.STORAGE_KEY = 'iconTheme';
__decorate([
    (0, inversify_1.inject)(NoneIconTheme),
    __metadata("design:type", NoneIconTheme)
], IconThemeService.prototype, "noneIconTheme", void 0);
__decorate([
    (0, inversify_1.postConstruct)(),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], IconThemeService.prototype, "init", null);
IconThemeService = IconThemeService_1 = __decorate([
    (0, inversify_1.injectable)()
], IconThemeService);
exports.IconThemeService = IconThemeService;
//# sourceMappingURL=icon-theme-service.js.map